@@ -1,13 +1,14 @@
 ;; -*- no-byte-compile: t; lexical-binding: nil -*-
-(define-package "copilot" "20250916.500"
+(define-package "copilot" "20251117.510"
   "An unofficial Copilot plugin."
-  '((emacs        "27.2")
-    (editorconfig "0.8.2")
-    (jsonrpc      "1.0.14")
-    (f            "0.20.0"))
+  '((emacs         "27.2")
+    (editorconfig  "0.8.2")
+    (jsonrpc       "1.0.14")
+    (f             "0.20.0")
+    (track-changes "1.4"))
   :url "https://github.com/copilot-emacs/copilot.el"
-  :commit "c42c18e346e1ff8399eb30d40527e593f614e4d3"
-  :revdesc "c42c18e346e1"
+  :commit "8e43edf1f3efe094ea42a21863b3b742a339742c"
+  :revdesc "8e43edf1f3ef"
   :keywords '("convenience" "copilot")
   :authors '(("zerol" . "z@zerol.me"))
   :maintainers '(("Jen-Chieh" . "jcs090218@gmail.com")
